<?php

class SMSApiClient
{
    private $client;

    private $config;

    private $sendername;

    private $normalize;

    private $httpClient;

    public function __construct()
    {
        $this->config = get_option('smsapi_client', array());

        $username = array_safe_get($this->config, 'api_username');
        $password = array_safe_get($this->config, 'api_password');
        $hostname = array_safe_get($this->config, 'api_hostname');

        $this->httpClient = new \SMSApi\Proxy\Http\Native($hostname);

        $this->client = new \SMSApi\Client($username);
        $this->client->setPasswordHash($password);
    }

    public function setNormalize($normalize)
    {
        $this->normalize = $normalize;
    }

    public function setSendername($sendername)
    {
        $this->sendername = $sendername;
    }

    public function getPhonebookGroups()
    {
        $actionsFactory = new SMSApi\Api\ContactsFactory($this->httpClient, $this->client);

        $groupListAction = $actionsFactory->actionGroupList();

        $response = $groupListAction->execute();

        return $response->getCollection();
    }

    public function getGroup($groupId)
    {
        $actionsFactory = new SMSApi\Api\ContactsFactory($this->httpClient, $this->client);

        $groupListAction = $actionsFactory->actionGroupGet($groupId);

        return $groupListAction->execute();
    }

    public function getSenderNames()
    {
        $actionsFactory = new SMSApi\Api\SenderFactory($this->httpClient, $this->client);

        $listAction = $actionsFactory->actionList();

        $response = $listAction->execute();
        $sendernames = $response->getList();

        return $sendernames ? $sendernames : array();
    }

    public function sendSms($to, $message)
    {
        $actionsFactory = new SMSApi\Api\SmsFactory($this->httpClient, $this->client);

        $actionSend = $actionsFactory->actionSend();

        $actionSend->setTo($to);
        $actionSend->setText($message);

        $settingsClient = get_option('smsapi_client');
        $defaultSendername = array_safe_get($settingsClient, 'api_sendername');

        $sendername = $this->sendername ? $this->sendername : $defaultSendername;

        if ($sendername) {
            $actionSend->setSender($sendername);
        }

        if ($this->normalize) {
            $actionSend->setNormalize($this->normalize);
        }

        return $actionSend->execute();
    }

    public function sendSmsToAll($message)
    {
        $actionsFactory = new SMSApi\Api\SmsFactory($this->httpClient, $this->client);

        $actionSend = $actionsFactory->actionSend();

        $actionSend->setText($message);

        $settingsClient = get_option('smsapi_client');

        $defaultSendername = array_safe_get($settingsClient, 'api_sendername');

        $sendername = $this->sendername ? $this->sendername : $defaultSendername;

        if ($sendername) {
            $actionSend->setSender($sendername);
        }

        $groupId = array_safe_get($settingsClient, 'phonebook_group');

        $group = $this->getGroup($groupId);

        $actionSend->setGroup($group->getName());

        if ($this->normalize) {
            $actionSend->setNormalize($this->normalize);
        }

        return $actionSend->execute();
    }

    public function saveSubscriber(array $subscriber)
    {
        $actionsFactory = new SMSApi\Api\ContactsFactory($this->httpClient, $this->client);

        $phoneNumber = array_safe_get($subscriber, 'phonenumber');

        $actionSave = $actionsFactory->actionContactAddFromPhoneNumber($phoneNumber);

        if ($birthdayDateStr = array_safe_get($subscriber, 'birthdate')) {
            $birthdayDate = new DateTime($birthdayDateStr);
            $actionSave->setBirthdayDate($birthdayDate);
        }

        $actionSave->setFirstName(array_safe_get($subscriber, 'firstname'));
        $actionSave->setLastName(array_safe_get($subscriber, 'lastname'));
        $actionSave->setEmail(array_safe_get($subscriber, 'email'));
        $actionSave->setCity(array_safe_get($subscriber, 'city'));

        $contact = $actionSave->execute();

        $clientSettings = get_option('smsapi_client', array());
        $defaultGroupId = array_safe_get($clientSettings, 'phonebook_group');

        $actionBindContactToGroup = $actionsFactory->actionContactGroupAdd($contact->getId(), $defaultGroupId);
        $actionBindContactToGroup->execute();

        return $contact;
    }

    public function updateSubscriber($subscriber, array $data)
    {
        $actionsFactory = new SMSApi\Api\ContactsFactory($this->httpClient, $this->client);

        $actionSave = $actionsFactory->actionContactEdit($subscriber->getId());

        if ($birthday = array_safe_get($data, 'birthdate')) {
            $birthdayDate = new DateTime($birthday);
            $actionSave->setBirthdayDate($birthdayDate);
        }

        $actionSave->setPhoneNumber(array_safe_get($data, 'phonenumber'));
        $actionSave->setFirstName(array_safe_get($data, 'firstname'));
        $actionSave->setLastName(array_safe_get($data, 'lastname'));
        $actionSave->setEmail(array_safe_get($data, 'email'));
        $actionSave->setCity(array_safe_get($data, 'city'));

        return $actionSave->execute();
    }

    public function overwriteSubscriber($subscriber, array $data)
    {
        $actionsFactory = new SMSApi\Api\ContactsFactory($this->httpClient, $this->client);

        $actionSave = $actionsFactory->actionContactEdit($subscriber->getId());

        if ($birthday = array_safe_get($data, 'birthdate')) {
            $birthdayDate = new DateTime($birthday);
            $actionSave->setBirthdayDate($birthdayDate);
        }

        if ($phonenumber = array_safe_get($data, 'phonenumber')) {
            $actionSave->setPhoneNumber($phonenumber);
        }

        if ($firstname = array_safe_get($data, 'firstname')) {
            $actionSave->setFirstName($firstname);
        }

        if ($lastname = array_safe_get($data, 'lastname')) {
            $actionSave->setLastName($lastname);
        }

        if ($email = array_safe_get($data, 'email')) {
            $actionSave->setEmail($email);
        }

        if ($city = array_safe_get($data, 'city')) {
            $actionSave->setCity($city);
        }

        return $actionSave->execute();
    }

    public function bindSubscriberToDefaultGroup($subscriber)
    {
        $actionsFactory = new SMSApi\Api\ContactsFactory($this->httpClient, $this->client);

        $clientSettings = get_option('smsapi_client', array());
        $defaultGroupId = array_safe_get($clientSettings, 'phonebook_group');

        $actionBindContactToGroup = $actionsFactory->actionContactGroupAdd($subscriber->getId(), $defaultGroupId);

        return $actionBindContactToGroup->execute();
    }

    public function getSubscriber($phonenumber)
    {
        $actionsFactory = new SMSApi\Api\ContactsFactory($this->httpClient, $this->client);

        $actionList = $actionsFactory->actionContactList();

        $actionList->setPhoneNumber($phonenumber);

        $clientSettings = get_option('smsapi_client', array());
        $defaultGroupId = array_safe_get($clientSettings, 'phonebook_group');

        $actionList->setGroupId($defaultGroupId);

        $contacts = $actionList->execute();

        $collection = $contacts->getCollection();

        return reset($collection);
    }

    public function getSubscriberFromAnyGroup($phonenumber)
    {
        $actionsFactory = new SMSApi\Api\ContactsFactory($this->httpClient, $this->client);

        $actionList = $actionsFactory->actionContactList();

        $actionList->setPhoneNumber($phonenumber);

        $contacts = $actionList->execute();

        $collection = $contacts->getCollection();

        return reset($collection);
    }

    public function getSubscriberById($contactId)
    {
        $actionsFactory = new SMSApi\Api\ContactsFactory($this->httpClient, $this->client);

        $actionGet = $actionsFactory->actionContactGet($contactId);

        return $actionGet->execute();
    }

    public function getAllSubscribers()
    {
        $actionsFactory = new SMSApi\Api\ContactsFactory($this->httpClient, $this->client);

        $actionList = $actionsFactory->actionContactList();

        $clientSettings = get_option('smsapi_client', array());
        $defaultGroupId = array_safe_get($clientSettings, 'phonebook_group');

        $actionList->setGroupId($defaultGroupId);

        $contacts = $actionList->execute();
        $collection = $contacts->getCollection();

        return $collection ? $collection : array();
    }

    public function getSubscribersList($limit, $offset, $search = null)
    {
        $actionsFactory = new SMSApi\Api\ContactsFactory($this->httpClient, $this->client);

        $actionList = $actionsFactory->actionContactList();

        $actionList->setOffsetAndLimit($offset, $limit);

        $clientSettings = get_option('smsapi_client', array());
        $defaultGroupId = array_safe_get($clientSettings, 'phonebook_group');

        $actionList->setGroupId($defaultGroupId);

        if ($search) {
            $actionList->setSearch($search);
        }

        $contacts = $actionList->execute();
        $collection = $contacts->getCollection();

        return $collection ? $collection : array();
    }

    public function removeSubscriber($contactId)
    {
        $actionsFactory = new SMSApi\Api\ContactsFactory($this->httpClient, $this->client);

        $actionContactGroupList = $actionsFactory->actionContactGroupList($contactId);

        $groupsResult = $actionContactGroupList->execute();
        $groups = $groupsResult->getCollection();

        if (count($groups) > 1) {
            $clientSettings = get_option('smsapi_client', array());
            $defaultGroupId = array_safe_get($clientSettings, 'phonebook_group');

            $action = $actionsFactory->actionGroupMemberDelete($defaultGroupId, $contactId);
        } else {
            $action = $actionsFactory->actionContactDelete($contactId);
        }

        return $action->execute();
    }

    public function removeSubscribers()
    {
        $actionsFactory = new SMSApi\Api\ContactsFactory($this->httpClient, $this->client);

        $clientSettings = get_option('smsapi_client', array());
        $defaultGroupId = array_safe_get($clientSettings, 'phonebook_group');

        $actionDelete = $actionsFactory->actionContactDeleteMultiple();
        $actionDelete->setGroupId($defaultGroupId);

        return $actionDelete->execute();
    }

    public function countContacts($search = null)
    {
        $actionsFactory = new SMSApi\Api\ContactsFactory($this->httpClient, $this->client);

        $actionList = $actionsFactory->actionContactList();

        $clientSettings = get_option('smsapi_client', array());
        $defaultGroupId = array_safe_get($clientSettings, 'phonebook_group');

        if ($search) {
            $actionList->setSearch($search);
        }

        $actionList->setGroupId($defaultGroupId);

        $result = $actionList->execute();

        return count($result->getCollection());
    }

    public function getSubscribersByIds(array $subscribersIds)
    {
        $actionsFactory = new SMSApi\Api\ContactsFactory($this->httpClient, $this->client);

        $actionList = $actionsFactory->actionContactList();

        $clientSettings = get_option('smsapi_client', array());
        $defaultGroupId = array_safe_get($clientSettings, 'phonebook_group');

        $actionList->setGroupId($defaultGroupId);

        $actionList->setIds($subscribersIds);

        $result = $actionList->execute();

        return $result->getCollection();
    }

    public function addGroup($groupName)
    {
        $actionsFactory = new SMSApi\Api\ContactsFactory($this->httpClient, $this->client);

        $actionAdd = $actionsFactory->actionGroupAdd($groupName);

        $group = $actionAdd->execute();

        return $group;
    }
}