<?php

require_once SMSAPI_PLUGIN_PATH . '/forms.php';
require_once SMSAPI_PLUGIN_PATH . '/classes/SmsapiPagination.php';

function action_subscribers_list()
{
    $smsapiClient = new SMSApiClientProxy();

    $searchText = array_safe_get($_GET, 'search_text');

    $count = $smsapiClient->countContacts($searchText);

    $uri = sprintf('admin.php?page=smsapi-subscribers%s', $searchText ? '&search_text=' . $searchText : '');

    $pagination = new SmsapiPagination(array(
        'base_url' => admin_url($uri),
        'total_items' => $count
    ));

    $subscribers = $smsapiClient->getSubscribersList($pagination->getLimit(), $pagination->getOffset(), $searchText);

    include_once SMSAPI_PLUGIN_PATH . "/views/admin/subscribers/list.php";
}

function action_subscribers_edit($subscriberId)
{
    $smsapiClient = new SMSApiClientProxy();

    $subscriber = $smsapiClient->getSubscriberById($subscriberId);

    if (!$subscriber) {
        wp_die('Subscriber not exists');
    }

    if (!empty($_POST)) {
        $form = new SubscriberEditForm($_POST);

        if ($form->isValid()) {
            $data = $form->getData();
            $subscriber = $smsapiClient->updateSubscriber($subscriber, $data);

            set_flash_success(__('Data was saved', 'smsapi'));
        } else {
            $errors = $form->getErrors();
            set_flash_error(__('Correct errors on the form and try again', 'smsapi'));
        }
    }

    include_once SMSAPI_PLUGIN_PATH . "/views/admin/subscribers/form.php";
}

function action_remove_one($subscriberId)
{
    $smsapiClient = new SMSApiClientProxy();

    $subscriber = $smsapiClient->getSubscriberById($subscriberId);

    if (!$subscriber) {
        wp_die('Subscriber not exists');
    }

    $smsapiClient->removeSubscriber($subscriberId);

    set_flash_success(__('Subscriber was removed', 'smsapi'));

    wp_redirect(admin_url() . 'admin.php?page=smsapi-subscribers');
}

function action_remove_by_ids(array $subscribersIds)
{
    $smsapiClient = new SMSApiClientProxy();

    foreach ($subscribersIds as $subscriberId) {
        $smsapiClient->removeSubscriber($subscriberId);
    }

    set_flash_success(__('Subscribers was removed', 'smsapi'));

    wp_redirect(admin_url() . 'admin.php?page=smsapi-subscribers');
}

function action_remove_all()
{
    $smsapiClient = new SMSApiClientProxy();

    $smsapiClient->removeSubscribers();

    set_flash_success(__('Subscribers was removed', 'smsapi'));

    wp_redirect(admin_url() . 'admin.php?page=smsapi-subscribers');
}

function action_sendsms(array $subscribersIds)
{
    $smsapiClient = new SMSApiClientProxy();

    $numbers = array();

    $subscribers = $smsapiClient->getSubscribersByIds($subscribersIds);

    foreach ($subscribers as $subscriber) {
        $numbers[] = $subscriber->getPhoneNumber();
    }

    session_set('gateway_recipients', $numbers);

    wp_redirect(admin_url() . 'admin.php?page=smsapi-send-sms');
}

function action_export(array $subscribersIds)
{
    $smsapiClient = new SMSApiClientProxy();

    $results = $smsapiClient->getSubscribersByIds($subscribersIds);

    if ($results) {
        $filepath = save_subscribers_to_file($results);

        send_file($filepath, $filepath, array(
            'delete' => true
        ));
    }
}

function action_export_all()
{
    $smsapiClient = new SMSApiClientProxy();

    $results = $smsapiClient->getAllSubscribers();

    if ($results) {
        $filepath = save_subscribers_to_file($results);

        send_file($filepath, $filepath, array(
            'delete' => true
        ));
    }
}

function save_subscribers_to_file($results)
{
    $filename = 'subscribers' . substr(sha1(uniqid()), 0, 6);
    $filepath = SMSAPI_TMP_PATH . '/' . $filename . '.csv';

    $fp = fopen($filepath, 'w');

    $headersRow = array('Numer', 'Imię', 'Nazwisko', 'Data urodzenia', 'Miasto', 'Email');

    fputcsv($fp, $headersRow);

    if ($results) {
        foreach ($results as $subscriber) {
            $birthday = $subscriber->getBirthdayDate() ?
                $subscriber->getBirthdayDate()->format('Y-m-d') : '';

            fputcsv($fp, array(
                $subscriber->getPhoneNumber(),
                $subscriber->getFirstName(),
                $subscriber->getLastName(),
                $birthday,
                $subscriber->getCity(),
                $subscriber->getEmail(),
            ));
        }
    }

    fclose($fp);

    return $filepath;
}